"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _jsonSchemaRefParser = _interopRequireDefault(require("json-schema-ref-parser"));

var _isArray = _interopRequireDefault(require("inspected/schema/is-array"));

var _isNil = _interopRequireDefault(require("inspected/schema/is-nil"));

var _isFunction = _interopRequireDefault(require("inspected/schema/is-function"));

var _isString = _interopRequireDefault(require("inspected/schema/is-string"));

var _isObject = _interopRequireDefault(require("inspected/schema/is-object"));

var _swagger2openapi = _interopRequireDefault(require("swagger2openapi"));

var _deepmerge = _interopRequireDefault(require("deepmerge"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const getParsed =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (basePath, dereference, parser, resolver, schema) {
    const refParserOptions = {
      parse: {
        parser
      },
      resolve: {
        custom: resolver
      }
    };

    if (!(0, _isArray.default)(dereference.mode)) {
      throw new Error('Dereference mode should be an array.');
    }

    if (!(0, _isFunction.default)(dereference.resolve)) {
      throw new Error('Dereference resolve should be a function.');
    }

    const params = (basePath ? [basePath] : []).concat([schema, refParserOptions]);
    const result = yield Promise.all(dereference.mode.map(
    /*#__PURE__*/
    function () {
      var _ref2 = _asyncToGenerator(function* (mode) {
        switch (mode) {
          case 'none':
            return yield _jsonSchemaRefParser.default.parse(...params);

          case 'external':
            return yield _jsonSchemaRefParser.default.bundle(...params);

          case 'all':
            return yield _jsonSchemaRefParser.default.dereference(...params);

          default:
            throw new Error(`Unknown dereference mode ${mode}.`);
        }
      });

      return function (_x6) {
        return _ref2.apply(this, arguments);
      };
    }()));
    return dereference.resolve(schema, result);
  });

  return function getParsed(_x, _x2, _x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();

const resolve = ({
  basePath,
  dereference,
  upgrade,
  parser,
  resolver
}) =>
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(function* (schema) {
    const parsed = yield getParsed(basePath, dereference, parser, resolver, schema);

    if (upgrade.enabled) {
      const upgraded = yield _swagger2openapi.default.convertObj(parsed, upgrade.options || {});
      return upgraded.openapi;
    }

    return parsed;
  });

  return function (_x7) {
    return _ref3.apply(this, arguments);
  };
}();

const parse = options =>
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(function* (schema) {
    if ((0, _isNil.default)(schema)) {
      throw new Error('No schema specified.');
    }

    if (!(0, _isString.default)(schema) && !(0, _isObject.default)(schema)) {
      throw new Error('Schema must be a string path or spec object.');
    } // TODO: validate options against schema


    if (!(0, _isNil.default)(options) && !(0, _isObject.default)(options)) {
      throw new Error('Options should be an object.');
    }

    const defaultOptions = {
      basePath: null,
      dereference: {
        mode: ['none'],
        resolve: (_, result) => result[0]
      },
      upgrade: {
        enabled: false
      },
      parser: {
        canParse: fileInfo => false,
        parse: function () {
          var _ref5 = _asyncToGenerator(function* (fileInfo) {});

          return function parse(_x9) {
            return _ref5.apply(this, arguments);
          };
        }()
      },
      resolver: {
        canResolve: fileInfo => false,
        resolve: function () {
          var _ref6 = _asyncToGenerator(function* (fileInfo) {});

          return function resolve(_x10) {
            return _ref6.apply(this, arguments);
          };
        }()
      }
    };
    const compiledOptions = options ? (0, _deepmerge.default)(defaultOptions, options) : defaultOptions;
    const parser = compiledOptions.parser ? {
      order: 1,
      canParse: fileInfo => {
        return compiledOptions.parser.canParse({
          path: fileInfo.url,
          extension: fileInfo.extension,
          data: fileInfo.data
        });
      },
      parse: function () {
        var _ref7 = _asyncToGenerator(function* (fileInfo) {
          return compiledOptions.parser.parse({
            path: fileInfo.url,
            extension: fileInfo.extension,
            data: fileInfo.data
          });
        });

        return function parse(_x11) {
          return _ref7.apply(this, arguments);
        };
      }()
    } : undefined;
    const resolver = compiledOptions.resolver ? {
      order: 1,
      canRead: fileInfo => {
        return compiledOptions.resolver.canResolve({
          path: fileInfo.url,
          extension: fileInfo.extension
        });
      },
      read: function () {
        var _ref8 = _asyncToGenerator(function* (fileInfo) {
          return yield compiledOptions.resolver.resolve({
            path: fileInfo.url,
            extension: fileInfo.extension
          });
        });

        return function read(_x12) {
          return _ref8.apply(this, arguments);
        };
      }()
    } : undefined;
    return yield resolve({
      basePath: compiledOptions.basePath,
      dereference: compiledOptions.dereference,
      upgrade: compiledOptions.upgrade,
      parser,
      resolver
    })(schema);
  });

  return function (_x8) {
    return _ref4.apply(this, arguments);
  };
}();

var _default = parse;
exports.default = _default;